@@ -18,6 +18,9 @@ use DateTime::Format::HTTP;
 use Imager;
 use URI::Escape 'uri_escape';
 use List::MoreUtils 'none';
+use WWW::Mechanize;
+use JSON::XS 'decode_json';
+use Data::asXML 0.07;
 
 use meon::Web::Form::Login;
 use meon::Web::Form::Delete;
@@ -34,6 +37,7 @@ sub auto : Private {
 
     meon::Web::env->clear;
     meon::Web::env->stash($c->stash);
+    meon::Web::env->session($c->session);
 
     my $uri      = $c->req->uri;
     my $hostname = $uri->host;
@@ -86,6 +90,7 @@ sub resolve_xml : Private {
     my ( $self, $c ) = @_;
 
     my $hostname_dir = $c->stash->{hostname_dir};
+    my $include_dir  = meon::Web::env->include_dir;
     my $path            =
         delete($c->session->{post_redirect_path})
         || $c->stash->{path}
@@ -141,6 +146,8 @@ sub resolve_xml : Private {
         $c->detach;
     }
 
+    meon::Web::env->xml_file($xml_file);
+
     unless (-e $xml_file) {
         my $not_found_handler = meon::Web::env->hostname_config->{'main'}{'not-found-handler'};
         if ($not_found_handler) {
@@ -152,12 +159,11 @@ sub resolve_xml : Private {
             $not_found_handler->check($content_dir, $relative_path);
         }
         $c->detach('/status_not_found', [($c->debug ? $path.' '.$xml_file : $path)])
-            unless -e $xml_file;
+            if (!eval { meon::Web::env->xml });
     }
 
     $xml_file = file($xml_file);
     $c->stash->{xml_file} = $xml_file;
-    meon::Web::env->xml_file($xml_file);
 
     my $dom = meon::Web::env->xml;
     my $xpc = meon::Web::Util->xpc;
@@ -165,6 +171,7 @@ sub resolve_xml : Private {
     $c->model('ResponseXML')->dom($dom);
 
     $c->model('ResponseXML')->push_new_element('current-path')->appendText($c->req->uri->path);
+    $c->model('ResponseXML')->push_new_element('current-uri')->appendText($c->req->uri->absolute);
     $c->model('ResponseXML')->push_new_element('static-mtime')->appendText(meon::Web::env->static_dir_mtime);
     $c->model('ResponseXML')->push_new_element('run-env')->appendText(Run::Env->current);
 
@@ -176,7 +183,38 @@ sub resolve_xml : Private {
         $user_el_username->appendText($c->user->username);
         $user_el->appendChild($user_el_username);
 
+        my $member = $c->member;
+        my $full_name_el = $c->model('ResponseXML')->create_element('full-name');
+        $full_name_el->appendText($member->get_member_meta('full-name'));
+        $user_el->appendChild($full_name_el);
+        my $profile_el = $c->model('ResponseXML')->create_element('profile');
+        if (my $member_profile = $member->get_member_meta_element('member-profile')) {
+            $profile_el->appendChild($member_profile);
+            $user_el->appendChild($profile_el);
+        }
+
+        $c->model('ResponseXML')->append_xml($user_el);
+
         my @user_roles = $c->user->roles;
+        if (my $backend_user_data = $c->session->{backend_user_data}) {
+            my $bu_data_el = $c->model('ResponseXML')->create_element('backend-user-data');
+            $c->model('ResponseXML')->append_xml($bu_data_el);
+            my $dxml = Data::asXML->new(
+                pretty    => Run::Env->dev,
+                namespace => 1,
+            );
+            $bu_data_el->appendChild(
+                $dxml->encode($backend_user_data)
+            );
+
+            if ($backend_user_data->{web_roles}) {
+                my @backed_roles = map {
+                    'backend-'.$_
+                } eval {@{$backend_user_data->{web_roles}}};
+                push(@user_roles, @backed_roles);
+            }
+        }
+
         my $roles_el = $c->model('ResponseXML')->create_element('roles');
         foreach my $role (@user_roles) {
             $roles_el->appendChild(
@@ -185,17 +223,11 @@ sub resolve_xml : Private {
         }
         $user_el->appendChild($roles_el);
 
-        my $member = $c->member;
-        my $full_name_el = $c->model('ResponseXML')->create_element('full-name');
-        $full_name_el->appendText($member->get_member_meta('full-name'));
-        $user_el->appendChild($full_name_el);
-
-        $c->model('ResponseXML')->append_xml($user_el);
-
         my @access_roles = map { $_->textContent } $xpc->findnodes('/w:page/w:meta/w:access/w:role',$dom);
-        if (@access_roles && (none { $_ ~~ \@user_roles } @access_roles)) {
-            $c->detach('/status_forbidden', []);
+        foreach my $role (@access_roles) {
+            $c->detach('/status_forbidden', []) if (none {$_ eq $role} @user_roles);
         }
+
     }
     else {
         if ($xpc->findnodes('/w:page/w:meta/w:members-only',$dom)) {
@@ -214,6 +246,55 @@ sub resolve_xml : Private {
         $c->detach;
     }
 
+    # includes
+    my (@include_elements) =
+        $xpc->findnodes('/w:page//w:include',$dom);
+    foreach my $include_el (@include_elements) {
+        my $include_path = $include_el->getAttribute('path');
+        unless ($include_path) {
+            $include_el->appendText('path attribute missing');
+            next;
+        }
+        my $include_rel = dir(meon::Web::Util->path_fixup($include_path));
+        my $file = file($include_dir, $include_rel)->absolute;
+        next unless -f $file;
+        $file = $file->resolve;
+        $c->detach('/status_forbidden', [])
+            unless $include_dir->contains($file);
+        my $include_xml = eval { XML::LibXML->load_xml(location => $file) };
+
+        my (@include_filter_elements) =
+            $xpc->findnodes('//w:apply-filter',$include_xml);
+        foreach my $include_filter_el (@include_filter_elements) {
+            my $filter_ident = $include_filter_el->getAttribute('ident');
+            die 'no filter name specified'
+                unless $filter_ident;
+            my $filter_class = 'meon::Web::Filter::'.$filter_ident;
+            load_class($filter_class);
+            my $status = $filter_class->new(
+                dom          => $include_xml,
+                include_node => $include_el,
+                user         => $c->user,
+            )->apply;
+            if (my $err_msg = $status->{error}) {
+                if (($status->{status} // 0) == 404) {
+                    $c->detach('/status_not_found', [$err_msg]);
+                }
+                else {
+                    die $err_msg;
+                }
+            }
+            $include_filter_el->parentNode->removeChild($include_filter_el);
+        }
+
+        if ($include_xml) {
+            $include_el->replaceNode($include_xml->documentElement());
+        }
+        else {
+            die 'failed to load include '.$@;
+        }
+    }
+
     # forms
     if (my ($form_el) = $xpc->findnodes('/w:page/w:meta/w:form',$dom)) {
         my $skip_form = 0;
@@ -503,7 +584,7 @@ sub _older_entries {
                     $dir->children(no_hidden => 1)
                 ;
             }
-            return $dir->relative($c->stash->{xml_file}->dir).'/';
+            return '/'.$dir->relative(meon::Web::env->content_dir).'/';
         }
 
         $cur_dir = $dir->basename;
@@ -538,7 +619,7 @@ sub _newer_entries {
                     $dir->children(no_hidden => 1)
                 ;
             }
-            return $dir->relative($c->stash->{xml_file}->dir).'/';
+            return '/'.$dir->relative(meon::Web::env->content_dir).'/';
         }
 
         $cur_dir = $dir->basename;
@@ -597,6 +678,34 @@ sub logout : Local {
 sub login : Local {
     my ( $self, $c ) = @_;
 
+    return $c->res->redirect($c->uri_for('/'))
+        if $c->user_exists;
+
+    my $members_folder = $c->default_auth_store->folder;
+
+    my $ext_auth_username = $c->session->{external_auth_username};
+    if (
+        meon::Web::env->hostname_config->{'auth'}{'external'}
+        && $ext_auth_username
+    ) {
+        my $member = meon::Web::Member->new(
+            members_folder => $members_folder,
+            username       => $ext_auth_username,
+        );
+
+        if ($member->exists) {
+            $c->set_authenticated($c->find_user({ username => $ext_auth_username }));
+            $c->log->info('user '.$ext_auth_username.' authenticated via external authentication');
+            $c->change_session_id;
+            delete $c->session->{external_auth_username};
+            return $c->res->redirect($c->req->uri->absolute);
+        }
+
+        my $registration_link = meon::Web::env->hostname_config->{'auth'}{'registration'};
+        $c->stash->{path} = $c->traverse_uri($registration_link);
+        $c->detach('resolve_xml', []);
+    }
+
     my $token    = $c->req->param('auth-token');
     my $username = $c->req->param('username');
     my $password = $c->req->param('password');
@@ -611,17 +720,14 @@ sub login : Local {
         return $c->res->redirect($c->uri_for($back_to));
     }
 
-    my $login_form = meon::Web::Form::Login->new(
-        action => $c->req->uri,
-    );
+    my $login_form = meon::Web::Form::Login->new();
 
     # token authentication
     if ($token) {
-        my $members_folder = $c->default_auth_store->folder;
         my $member;
         if (($token eq 'admin') && $c->user_exists) {
             my @roles = $c->user->roles;
-            if ('admin' ~~ \@roles) {
+            if (any {$_ eq 'admin'} @roles) {
                 $member = meon::Web::Member->new(
                     members_folder => $members_folder,
                     username       => $username,
@@ -658,29 +764,92 @@ sub login : Local {
     }
     else {
         $login_form->process(params=>$c->req->params);
-        if ($username =~ m/\@/) {
-            my $members_folder = $c->default_auth_store->folder;
-            my $member = meon::Web::Member->find_by_email(
-                members_folder => $members_folder,
-                email          => $username,
-            );
-            $username = $member->user->username
-                if $member;
-        }
-        if ($username && $password && $login_form->is_valid) {
-            if (
-                $c->authenticate({
-                    username => $username,
-                    password => $password,
-                })
-            ) {
-                $c->log->info('user '.$username.' authenticated');
-                $c->change_session_id;
-                return $c->res->redirect($c->req->uri);
+
+        if (meon::Web::env->hostname_config->{'auth'}{'external'}) {
+            if ($username && $password && $login_form->is_valid) {
+                my $auth_url       = meon::Web::env->hostname_config->{'auth'}{'url'};
+                my $username_field = meon::Web::env->hostname_config->{'auth'}{'username'};
+                my $password_field = meon::Web::env->hostname_config->{'auth'}{'password'};
+                my $content_match  = meon::Web::env->hostname_config->{'auth'}{'content-match'};
+
+                my $mech = WWW::Mechanize->new();
+                if ($content_match) {
+                    eval {
+                        $mech->get( $auth_url );
+                        $mech->submit_form(
+                            with_fields      => {
+                                $username_field => $username,
+                                $password_field => $password,
+                            }
+                        );
+                        die 'external auth failed - status '.$mech->status
+                            unless $mech->status == 200;
+                        $mech->get( $auth_url );
+                        if ($content_match) {
+                            die 'external auth failed - content does not match m/'.$content_match.'/xms ('.$mech->uri.')'
+                                unless $mech->content =~ m/$content_match/xms;
+                        }
+                    };
+                }
+                else {
+                    eval {
+                        my $res = $mech->post( $auth_url, {
+                            $username_field => $username,
+                            $password_field => $password,
+                        });
+                        die 'external auth failed - status '.$mech->status
+                            unless $mech->status == 200;
+                        if ($res->header('Content-Type') =~ m{application/json}) {
+                            my $data = eval { decode_json($res->content) };
+                            if ($data && $data->{user_data}) {
+                                $c->session->{backend_user_data} = $data->{user_data};
+                            }
+                        }
+                    };
+                }
+                if ($@) {
+                    $login_form->field('password')->add_error('authentication failed, please check your password or try again later');
+                    $c->log->error($@);
+                    $c->res->status(403);
+                }
+                else {
+                    $c->session->{external_auth_username} = $username;
+                    if (my $user = $c->find_user({ username => $username })) {
+                        $c->set_authenticated($user);
+                        delete $c->session->{external_auth_username};
+                    }
+                    $c->log->info('user '.$username.' authenticated via external authentication [2]');
+                    return $c->res->redirect(
+                        $c->req->uri_with({username => undef, password => undef})->absolute
+                    );
+                }
             }
-            else {
-                $c->log->info('login of user '.$username.' fail');
-                $login_form->field('password')->add_error('authentication failed');
+        }
+        else {
+            if ($username =~ m/\@/) {
+                my $member = meon::Web::Member->find_by_email(
+                    members_folder => $members_folder,
+                    email          => $username,
+                );
+                $username = $member->user->username
+                    if $member;
+            }
+            if ($username && $password && $login_form->is_valid) {
+                if (
+                    $c->authenticate({
+                        username => $username,
+                        password => $password,
+                    })
+                ) {
+                    $c->log->info('user '.$username.' authenticated');
+                    $c->change_session_id;
+                    return $c->res->redirect($c->req->uri);
+                }
+                else {
+                    $c->log->info('login of user '.$username.' fail');
+                    $login_form->field('password')->add_error('authentication failed');
+                    $c->res->status(403);
+                }
             }
         }
     }
